/* Simple Plugin API */
/* SPDX-FileCopyrightText: Copyright © 2018 Wim Taymans */
/* SPDX-License-Identifier: MIT */

#ifndef SPA_PARAM_TYPE_INFO_H
#define SPA_PARAM_TYPE_INFO_H

#include <spa/param/param-types.h>
#include <spa/param/buffers-types.h>
#include <spa/param/props-types.h>
#include <spa/param/format-types.h>
#include <spa/param/latency-types.h>
#include <spa/param/port-config-types.h>
#include <spa/param/profiler-types.h>
#include <spa/param/profile-types.h>
#include <spa/param/route-types.h>
#include <spa/param/tag-types.h>

#endif /* SPA_PARAM_TYPE_INFO_H */
