/* Simple Plugin API */
/* SPDX-FileCopyrightText: Copyright © 2018 Wim Taymans */
/* SPDX-License-Identifier: MIT */

#ifndef SPA_AUDIO_TYPES_H
#define SPA_AUDIO_TYPES_H

#include <spa/param/audio/raw-types.h>
#include <spa/param/audio/iec958-types.h>
#include <spa/param/audio/mp3-types.h>
#include <spa/param/audio/aac-types.h>
#include <spa/param/audio/wma-types.h>
#include <spa/param/audio/amr-types.h>

#endif /* SPA_AUDIO_TYPES_H */
